###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: config.tcl,v 1.18 2001/06/07 08:19:26 tothwolf Exp $
##

# name: config
# version: 1.2.0.0
# config: config.conf
# description: Provides a dcc command called `.wpconf' that allows \
#	interactive configuration of module settings from eggdrop's \
#	partyline.
# requires: getModuleDatabaseData wpLog listModules listMaxElementLength \
#	moduleExists moduleLoaded moduleLoad moduleUnload moduleConfig \
#	moduleConfigList moduleData moduleDataList splitList \
#	saveModuleDatabase loadModuleDatabase moveFields whoisFields

namespace eval config {

  ##
  ## Configure module settings: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:wpconf {hand idx arg} {
    variable NamespaceCurrent
    variable configData
    variable IdxData

    # First check if there is a stored config module
    if {([string compare "" [set module [getuser $hand XTRA $configData(savefield)]]]) && \
        ([string compare "" [getModuleDatabaseData $module config]])} then {
      set IdxData($idx) $module
    } else {
      # Find first module that has config options and use it as the default
      foreach module [listModules 1] {
        if {[string compare "" [getModuleDatabaseData $module config]]} then {
          set IdxData($idx) $module
          setuser $hand XTRA [string toupper $configData(savefield)] $module
          break
        }
      }
    }
    # Fall back to 'wp' if no modules with config options were found
    if {![info exists IdxData($idx)]} then {
      set IdxData($idx) wp
    }
    control $idx ${NamespaceCurrent}::control:config
    putdcc $idx "Entering wolfpack configuration system..."
    putdcc $idx "Type 'help' for help."
    putdcc $idx "$IdxData($idx)>"
    return 1
  }

  ##
  ## Configure module settings: control
  ##
  ## Args:    idx, text
  ## Returns: 0 during command loop,
  ##          1 to exit back to partyline
  ##
  proc control:config {idx text} {
    variable IdxData

    set text [split [string trimright $text]]
    set log 1
    switch -exact -- [string tolower [string trimleft [lindex $text 0] .]] {
      set {
        ConfigSet $idx $text
      }
      mod {
        ConfigMod $idx $text
      }
      opt {
        ConfigOpt $idx $text
      }
      list {
        ConfigList $idx $text
      }
      save {
        ConfigSave $idx $text
      }
      enable {
        ConfigEnable $idx $text
      }
      disable {
        ConfigDisable $idx $text
      }
      modules {
        ConfigModules $idx $text
      }
      info {
        ConfigInfo $idx $text
      }
      help {
        ConfigHelp $idx $text
      }
      quit {
        putdcc $idx "*** Ja mata!"
        putdcc $idx "Returning you to command mode..."
        unset IdxData($idx)
        return 1
      }
      default {
        set log 0
        putdcc $idx "What?  You need 'help'"
      }
    }
    if {$log} then {
      wpLog c * "#[idx2hand $idx]# $text"
    }
    putdcc $idx "$IdxData($idx)>"
    return 0
  }

  ##
  ## Configure module settings: control: set
  ##
  ## Args:    idx, text
  ## Returns: nothing
  ##
  proc ConfigSet {idx text} {
    variable NamespaceParent
    variable IdxData

    set defaultModule $IdxData($idx)
    set tmp [split [lindex $text 1] :]
    set tmpModule [lindex $tmp 0]
    set tmpOption [lindex $tmp 1]
    set tmpWhat [lrange $text 2 end]
    if {(![string compare "" $tmpModule]) || \
        (([moduleExists $tmpModule]) && \
         (![string compare "" $tmpOption]) && \
         ([string compare $defaultModule $tmpModule]))} then {
      putdcc $idx "Usage: set <module>:<option> \[what\] | <option> \[what\]"
    } elseif {[info exists ${NamespaceParent}::${tmpModule}::configData($tmpOption)]} then {
      set module $tmpModule
      set option $tmpOption
      set what $tmpWhat
    } elseif {[info exists ${NamespaceParent}::${defaultModule}::configData($tmpModule)]} then {
      set module $defaultModule
      set option $tmpModule
      set what $tmpWhat
    } elseif {[moduleExists $tmpModule]} then {
      if {![string compare "" [getModuleDatabaseData $tmpModule config]]} then {
        putdcc $idx "Error: no config options for module: $tmpModule"
      } elseif {([info exists ${NamespaceParent}::${tmpModule}::configData($tmpOption)]) && \
                (![moduleLoaded $tmpModule])} then {
        putdcc $idx "Error: module: $tmpModule is not loaded"
      } elseif {(![info exists ${NamespaceParent}::${tmpModule}::configData($tmpOption)]) && \
                ([string compare $defaultModule $tmpModule])} then {
        putdcc $idx "Error: unknown option for module: $tmpModule: $tmpOption"
      } elseif {(![info exists ${NamespaceParent}::${tmpModule}::configData($tmpOption)])} then {
        putdcc $idx "Error: unknown module option: $tmpOption"
      }
    } elseif {![info exists ${NamespaceParent}::${defaultModule}::configData($tmpModule)]} then {
      putdcc $idx "Error: unknown module or option: $tmpModule"
    }
    if {([info exists module]) && ([info exists option])} then {
      if {![string compare "" $what]} then {
        putdcc $idx "Currently: [set ${NamespaceParent}::${module}::configData($option)]"
      } else {
        if {[string compare $what [set ${NamespaceParent}::${module}::configData($option)]]} then {
          set ${NamespaceParent}::${module}::configData($option) $what
          set ${NamespaceParent}::${module}::configDataChanged 1
          if {![string compare $defaultModule $module]} then {
            putdcc $idx "Set $option to: $what"
          } else {
            putdcc $idx "Set $module:$option to: $what"
          }
        } else {
          putdcc $idx "Error: $option was already set to: $what"
        }
      }
    }
    return
  }

  ##
  ## Configure module settings: control: mod
  ##
  ## Args:    idx, text
  ## Returns: nothing
  ##
  proc ConfigMod {idx text} {
    variable IdxData
    variable configData

    set module [lindex $text 1]
    if {![string compare "" $module]} then {
      putdcc $idx "Usage: mod <module>"
    } else {
      if {[moduleExists $module]} then {
        if {[moduleLoaded $module]} then {
          if {[string compare "" [getModuleDatabaseData $module config]]} then {
            set IdxData($idx) $module
            set hand [idx2hand $idx]
            if {[string compare [string tolower $module] \
                 [string tolower [getuser $hand XTRA $configData(savefield)]]]} then {
              setuser $hand XTRA [string toupper $configData(savefield)] $module
            }
            putdcc $idx "Default module now: $module"
          } else {
            putdcc $idx "Error: no config options for module: $module"
          }
        } else {
          putdcc $idx "Error: module: $module is not loaded"
        }
      } else {
        putdcc $idx "Error: unknown module: $module"
      }
    }
    return
  }

  ##
  ## Configure module settings: control: opt
  ##
  ## Args:    idx, text
  ## Returns: nothing
  ##
  proc ConfigOpt {idx text} {
    variable NamespaceParent
    variable IdxData

    set module [lindex $text 1]
    if {![string compare "" $module]} then {
      set module $IdxData($idx)
    }
    if {[moduleExists $module]} then {
      if {[moduleLoaded $module]} then {
        if {[string compare "" [getModuleDatabaseData $module config]]} then {
          putdcc $idx "Module option info for module: $module"
          putdcc $idx "Option       Info"
          foreach option [lsort [array names ${NamespaceParent}::${module}::configData]] {
            if {[info exists ${NamespaceParent}::${module}::configDataDesc($option)]} then {
              putdcc $idx [format "%-12s %s" $option [set ${NamespaceParent}::${module}::configDataDesc($option)]]
            } else {
              putdcc $idx [format "%-12s %s" $option "(no info)"]
            }
          }
        } else {
          putdcc $idx "Error: no config options for module: $module"
        }
      } else {
        putdcc $idx "Error: module: $module is not loaded"
      }
    } else {
      putdcc $idx "Error: unknown module: $module"
    }
    return
  }

  ##
  ## Configure module settings: control: list
  ##
  ## Args:    idx, text
  ## Returns: nothing
  ##
  proc ConfigList {idx text} {
    variable NamespaceParent
    variable IdxData

    set module [lindex $text 1]
    if {![string compare "" $module]} then {
      set module $IdxData($idx)
    }
    if {![string compare "*" $module]} then {
      set modules [listModules]
      if {[string compare "" $modules]} then {
        putdcc $idx "Module settings for all modules:"
        putdcc $idx "  Option       Value"
        foreach module $modules {
          if {[moduleLoaded $module]} then {
            putdcc $idx "Module: $module"
            if {[string compare "" [getModuleDatabaseData $module config]]} then {
              foreach option [lsort [array names ${NamespaceParent}::${module}::configData]] {
                putdcc $idx [format "  %-12s  %s" $option [set ${NamespaceParent}::${module}::configData($option)]]
              }
            } else {
              putdcc $idx "  (no settings)"
            }
          }
        }
      } else {
        putdcc $idx "No modules loaded."
      }
    } else {
      if {[moduleExists $module]} then {
        if {[moduleLoaded $module]} then {
          if {[string compare "" [getModuleDatabaseData $module config]]} then {
            putdcc $idx "Module settings for module: $module"
            putdcc $idx "Option       Value"
            foreach option [lsort [array names ${NamespaceParent}::${module}::configData]] {
              putdcc $idx [format "%-12s %s" $option [set ${NamespaceParent}::${module}::configData($option)]]
            }
          } else {
            putdcc $idx "Error: no config options for module: $module"
          }
        } else {
          putdcc $idx "Error: module: $module is not loaded"
        }
      } else {
        putdcc $idx "Error: unknown module: $module"
      }
    }
    return
  }

  ##
  ## Configure module settings: control: save
  ##
  ## Args:    idx, text
  ## Returns: nothing
  ##
  proc ConfigSave {idx text} {
    variable NamespaceParent

    set module [lindex $text 1]
    if {[set ${NamespaceParent}::moduleDatabaseDataChanged]} then {
      putdcc $idx "Saving module database"
      saveModuleDatabase 1
    }
    if {![string compare "" $module]} then {
      putdcc $idx "Saving changed config data for all modules"
      moduleConfigList save * 0 1
    } else {
      if {[moduleExists $module]} then {
        if {[moduleLoaded $module]} then {
          if {[string compare "" [getModuleDatabaseData $module config]]} then {
            putdcc $idx "Saving changed config data for module: $module"
            moduleConfig save $module 0 1
          } else {
            putdcc $idx "Error: no config options for module: $module"
          }
        } else {
          putdcc $idx "Error: module: $module is not loaded"
        }
      } else {
        putdcc $idx "Error: unknown module: $module"
      }
    }
    return
  }

  ##
  ## Configure module settings: control: enable
  ##
  ## Args:    idx, text
  ## Returns: nothing
  ##
  proc ConfigEnable {idx text} {
    set module [lindex $text 1]
    if {![string compare "" $module]} then {
      putdcc $idx "Usage: enable <module>"
    } else {
      if {[moduleExists $module]} then {
        if {![moduleLoaded $module]} then {
          putdcc $idx "Loading module: $module"
          if {[catch {moduleLoad $module} result]} then {
            putdcc $idx "Error loading $module: $result"
            wpLog o * "Error loading $module: $result"
          }
        } else {
          putdcc $idx "Error: $module is already enabled"
        }
      } else {
        putdcc $idx "Error: unknown module: $module"
      }
    }
    return
  }

  ##
  ## Configure module settings: control: disable
  ##
  ## Args:    idx, text
  ## Returns: nothing
  ##
  proc ConfigDisable {idx text} {
    set module [lindex $text 1]
    if {![string compare "" $module]} then {
      putdcc $idx "Usage: disable <module>"
    } else {
      if {[moduleExists $module]} then {
        if {[moduleLoaded $module]} then {
          putdcc $idx "Disabling module: $module"
          if {[catch {moduleUnload $module} result]} then {
            putdcc $idx "Error unloading $module: $result"
            wpLog o * "Error unloading $module: $result"
          }
        } else {
          putdcc $idx "Error: $module is already disabled"
        }
      } else {
        putdcc $idx "Error: unknown module: $module"
      }
    }
    return
  }

  ##
  ## Configure module settings: control: modules
  ##
  ## Args:    idx, text
  ## Returns: nothing
  ##
  proc ConfigModules {idx text} {
    set modules [listModules]
    set fmtlen [listMaxElementLength $modules]
    if {$fmtlen < 6} then {
      set fmtlen 6 ;# 'Module'
    }
    putdcc $idx "[format "%-${fmtlen}s  %-18s  %s" Module Version Loaded]"
    foreach module $modules {
      set version [getModuleDatabaseData $module version]
      set load [getModuleDatabaseData $module load]
      putdcc $idx "[format "%-${fmtlen}s  %-18s  %s" $module $version $load]"
    }
    return
  }

  ##
  ## Configure module settings: control: info
  ##
  ## Args:    idx, text
  ## Returns: nothing
  ##
  proc ConfigInfo {idx text} {
    variable IdxData

    set module [lindex $text 1]
    if {![string compare "" $module]} then {
      set module $IdxData($idx)
    }
    if {![string compare "*" $module]} then {
      putdcc $idx "Module info for all modules:"
      foreach module [listModules] {
        putdcc $idx $module
        foreach line [splitList [getModuleDatabaseData $module description] 65 " " " "] {
          putdcc $idx "  $line"
        }
      }
    } else {
      if {[moduleExists $module]} then {
        putdcc $idx "Module info for module: $module"
        foreach line [splitList [getModuleDatabaseData $module description] 65 " " " "] {
          putdcc $idx "  $line"
        }
      } else {
        putdcc $idx "Error: unknown module: $module"
      }
    }
    return
  }

  ##
  ## Configure module settings: control: help
  ##
  ## Args:    idx, text
  ## Returns: nothing
  ##
  proc ConfigHelp {idx text} {
    set what [lindex $text 1]
    if {![string compare "" $what]} then {
      set what help
    }
    switch -exact -- $what {
      set {
        putdcc $idx "###  set <module>:<option> \[what\] | <option> \[what\]"
        putdcc $idx "   Sets a module option to what you specify."
        putdcc $idx "   Shows current setting if no new value is given."
      }
      mod {
        putdcc $idx "###  mod <module>"
        putdcc $idx "   Changes your default module to the given module."
      }
      opt {
        putdcc $idx "###  opt \[module\]"
        putdcc $idx "   Shows options avaliable for the given module,"
        putdcc $idx "   or your default module if none is given."
      }
      list {
        putdcc $idx "###  list \[module|*\]"
        putdcc $idx "   Shows settings and their current values for"
        putdcc $idx "   the given module, all settings for all modules"
        putdcc $idx "   if '*' is given, or all settings for your"
        putdcc $idx "   default module if none is given."
      }
      save {
        putdcc $idx "###  save \[module\]"
        putdcc $idx "   Saves config info for the given module,"
        putdcc $idx "   or all modules if none is given."
      }
      enable {
        putdcc $idx "###  enable <module>"
        putdcc $idx "   Enable and load the given module."
      }
      disable {
        putdcc $idx "###  disable <module>"
        putdcc $idx "   Disable  and unload the given module."
      }
      modules {
        putdcc $idx "###  modules"
        putdcc $idx "   Shows version and decription for all modules"
      }
      info {
        putdcc $idx "###  info \[module|*\]"
        putdcc $idx "   Shows description for the given module,"
        putdcc $idx "   or for all modules if '*' is given,"
        putdcc $idx "   or default module if none is given."
      }
      help {
        putdcc $idx "Commands for wpconf:"
        putdcc $idx "  set       <module>:<option> \[what\] | <option> \[what\]"
        putdcc $idx "  mod       <module>"
        putdcc $idx "  opt       \[module\]"
        putdcc $idx "  list      \[module|*\]"
        putdcc $idx "  save      \[module\]"
        putdcc $idx "  enable    <module>"
        putdcc $idx "  disable   <module>"
        putdcc $idx "  modules"
        putdcc $idx "  info      \[module|*\]"
        putdcc $idx "  help      \[command\]"
        putdcc $idx "  quit"
        putdcc $idx "You can get help on individual commands: 'help <command>'"
      }
      quit {
        putdcc $idx "###  quit"
        putdcc $idx "  Quits the wpconf system and returns you to the partyline."
      }
      default {
        putdcc $idx "No help available on that."
      }
    }
    return
  }

  ##
  ## Save module config and data files: dcc
  ##
  ## Args:    handle, idx, arg {module}
  ## Returns: 1
  ##
  proc dcc:save {hand idx arg} {
    variable NamespaceParent

    set what [lindex [split [string trimright $arg]] 0]
    if {![string compare "" $what]} then {
      *dcc:save $hand $idx $arg
      if {[set ${NamespaceParent}::moduleDatabaseDataChanged]} then {
        putdcc $idx "Saving module database"
        saveModuleDatabase 1
      }
      moduleConfigList save * 0 1
      moduleDataList save * 0 1
      return 0
    } elseif {[moduleExists $what]} then {
      if {[moduleLoaded $what]} then {
        if {[set ${NamespaceParent}::moduleDatabaseDataChanged]} then {
          putdcc $idx "Saving module database"
          saveModuleDatabase 1
        }
        moduleConfig save $what 1 1
        moduleData save $what 1 1
      } else {
        putdcc $idx "Error: module: $what is not loaded"
      }
    } else {
      putdcc $idx "Error: unknown module: $what"
    }
    return 1
  }

  ##
  ## Reload module config and data files: dcc
  ##
  ## Args:    handle, idx, arg {module}
  ## Returns: 1
  ##
  proc dcc:reload {hand idx arg} {
    set what [lindex [split [string trimright $arg]] 0]
    if {![string compare "" $what]} then {
      *dcc:reload $hand $idx $arg
      loadModuleDatabase 1
      moduleConfigList load * 0 1
      moduleDataList load * 0 1
      return 0
    } elseif {[moduleExists $what]} then {
      if {[moduleLoaded $what]} then {
        loadModuleDatabase 1
        moduleConfig load $what 1 1
        moduleData load $what 1 1
      } else {
        putdcc $idx "Error: module: $what is not loaded"
      }
    } else {
      putdcc $idx "Error: unknown module: $what"
    }
    return 1
  }

  ##
  ## Save changed module config and data files once an hour: time
  ##
  ## Args:    handle, idx, arg {module}
  ## Returns: 1
  ##
  proc time:hourly {minute hour day month year} {
    variable NamespaceParent

    if {[set ${NamespaceParent}::moduleDatabaseDataChanged]} then {
      saveModuleDatabase 1
    }
    moduleConfigList save * 0 1
    moduleDataList save * 0 1
    return 0
  }

  ##
  ## Backup module data files every 24 hours: time
  ##
  ## Args:    handle, idx, arg {module}
  ## Returns: 1
  ##
  proc time:daily {minute hour day month year} {
    moduleDataList backup * 0 1
    return 0
  }

  ##
  ## Save changed module config and data files: evnt
  ##
  ## Args:    handle, idx, arg {module}
  ## Returns: 1
  ##
  proc evnt:save {type} {
    moduleConfigList save * 0 1
    moduleDataList save * 0 1
    return 0
  }

  ##
  ## Trace configuration changes
  ##
  ## Args:    name1, name2, operation
  ## Returns: nothing
  ##
  proc TraceConfigData {name1 name2 op} {
    variable configData
    variable oldConfigData

    if {[string compare $oldConfigData($name2) $configData($name2)]} then {
      switch -exact -- $name2 {
        savefield {
          moveFields $oldConfigData($name2) [string toupper $configData($name2)]
          whoisFields update $configData($name2) $oldConfigData($name2)
        }
      }
      set oldConfigData($name2) $configData($name2)
    }
    return
  }

  ##
  ## Trace and respond to variable changes
  ##
  ## Args:    action {variable|vdelete}
  ## Returns: nothing
  ##
  proc TraceTable {action} {
    variable NamespaceCurrent
    variable configData

    trace $action configData w ${NamespaceCurrent}::TraceConfigData
    return
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable NamespaceParent
    variable NamespaceCurrent
    variable configData
    variable oldConfigData

    switch -exact -- $action {
      load {
        if {![info exists NamespaceParent]} then {
          set NamespaceParent [namespace parent]
        }
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
        foreach {name data} [array get configData] {
          if {![info exists oldConfigData($name)]} then {
            set oldConfigData($name) $data
          }
        }
        whoisFields update $configData(savefield)
      }
      unload {
        whoisFields remove $configData(savefield)
      }
    }
    return
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    global numversion
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "dcc time evnt"
    }
    foreach type $what {
      switch -exact -- $type {
        dcc {
          $action dcc n|- wpconf ${NamespaceCurrent}::dcc:wpconf
          $action dcc m|- save ${NamespaceCurrent}::dcc:save
          $action dcc m|- reload ${NamespaceCurrent}::dcc:reload
        }
        time {
          $action time -|- "00 * * * *" ${NamespaceCurrent}::time:hourly
          $action time -|- "00 00 * * *" ${NamespaceCurrent}::time:daily
        }
        evnt {
          catch {set numversion}
          # earlier eggdrop versions don't have evnt bindings
          if {$numversion >= 1032800} then {
            $action evnt -|- save ${NamespaceCurrent}::evnt:save
          }
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    savefield wpconf
  }

  # Configuration data descriptions
  array set configDataDesc {
    savefield "\[field\] field used to store last module configured"
  }

} ;# namespace config

return
