###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: faqserv.tcl,v 1.3 2001/06/01 05:38:49 tothwolf Exp $
##

# name: faqserv
# version: 1.6.5.3
# config: faqserv.conf
# description:
# requires: this_module_needs_to_be_converted

namespace eval faqserv {

  # Set namespace variables
  set NamespaceParent [namespace parent]
  set NamespaceCurrent [namespace current]

  # set data information
  set data(faqdata) "1 $conf(faqdatafile) ${NamespaceCurrent}::faqdata"

  # load data
  module_data load "" 1 0 "1 $conf(faqdatafile) ${NamespaceCurrent}::faqdata"

  # show the given faq topic; pub
  proc pub:faq {nick uhost hand chan arg} {
    return [faq pub putmsg $chan putnot $nick $nick $uhost $hand $chan $arg]
  }

  # show the given faq topic; msg
  proc msg:faq {nick uhost hand arg} {
    return [faq msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # show the given faq topic; dcc
  proc dcc:faq {hand idx arg} {
    return [faq dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  # show the given faq topic
  proc faq {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    variable faqdata
    variable NamespaceCurrent
    set what [lindex [split [string trimright $arg]] 0]
    if {![string compare "" $what]} then {
      put $resp2 $dest2 "Usage: [command_char $type]faq <topic>"
    } else {
      set topic [arrayFindElementName ${NamespaceCurrent}::faqdata $what]
      if {[string compare "" $topic]} then {
        put $resp1 $dest1 "Info for $topic: [join $faqdata($topic)]"
      } else {
        put $resp2 $dest2 "Faq topic $what does not exist."
      }
    }
    return 1
  }

  # add a faq topic to the faq list; pub
  proc pub:addfaq {nick uhost hand chan arg} {
    return [addfaq pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
  }

  # add a faq topic to the faq list; msg
  proc msg:addfaq {nick uhost hand arg} {
    return [addfaq msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # add a faq topic to the faq list; dcc
  proc dcc:addfaq {hand idx arg} {
    return [addfaq dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  # add a faq topic to the faq list
  proc addfaq {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    variable faqdata
    variable faqdata_changed
    variable NamespaceCurrent
    if {[check_auth $type $nick $uhost $hand $chan addfaq]} then {
      set arg [split [string trimright $arg]]
      set what [lindex $arg 0]
      set content [join [lrange $arg 1 end]]
      if {(![string compare "" $what]) || (![string compare "" $content])} then {
        put $resp2 $dest2 "Usage: [command_char $type]addfaq <topic> <content>"
      } else {
        set topic [arrayFindElementName ${NamespaceCurrent}::faqdata $what]
        if {[string compare "" $topic]} then {
          set faqdata($topic) [list $content]
          set faqdata_changed 1
          put $resp1 $dest1 "Added faq topic: $topic"
        } else {
          put $resp2 $dest2 "Faq $topic already exists."
        }
      }
      return 1
    }
    return 0
  }

  # remove a faq topic from the faq list; pub
  proc pub:delfaq {nick uhost hand chan arg} {
    return [delfaq pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
  }

  # remove a faq topic from the faq list; msg
  proc msg:delfaq {nick uhost hand arg} {
    return [delfaq msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # remove a faq topic from the faq list; dcc
  proc dcc:delfaq {hand idx arg} {
    return [delfaq dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  # remove a faq topic from the faq list
  proc delfaq {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    variable faqdata
    variable faqdata_changed
    variable NamespaceCurrent
    if {[check_auth $type $nick $uhost $hand $chan delfaq]} then {
      set what [lindex [split [string trimright $arg]] 0]
      if {![string compare "" $what]} then {
        put $resp2 $dest2 "Usage: [command_char $type]delfaq <topic>"
      } else {
        set topic [arrayFindElementName ${NamespaceCurrent}::faqdata $what]
        if {[string compare "" $topic]} then {
          unset faqdata($topic)
          set faqdata_changed 1
          put $resp1 $dest1 "Removed faq topic: $topic"
        } else {
          put $resp2 $dest2 "Faq $what does not exist."
        }
      }
      return 1
    }
    return 0
  }

  # change a faq topic in the faq list; pub
  proc pub:chfaq {nick uhost hand chan arg} {
    return [chfaq pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
  }

  # change a faq topic in the faq list; msg
  proc msg:chfaq {nick uhost hand arg} {
    return [chfaq msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # change a faq topic in the faq list; dcc
  proc dcc:chfaq {hand idx arg} {
    return [chfaq dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  # change a faq topic in the faq list
  proc chfaq {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    variable faqdata
    variable faqdata_changed
    variable NamespaceCurrent
    if {[check_auth $type $nick $uhost $hand $chan addfaq]} then {
      set arg [split [string trimright $arg]]
      set what [lindex $arg 0]
      set content [join [lrange $arg 1 end]]
      if {(![string compare "" $what]) || (![string compare "" $content])} then {
        put $resp2 $dest2 "Usage: [command_char $type]chfaq <topic> <content>"
      } else {
        set topic [arrayFindElementName ${NamespaceCurrent}::faqdata $what]
        if {[string compare "" $topic]} then {
          set faqdata($topic) [list $content]
          set faqdata_changed 1
          put $resp1 $dest1 "Changed faq topic: $topic"
        } else {
          put $resp2 $dest2 "Faq $what does not exist."
        }
      }
      return 1
    }
    return 0
  }

  # answer a chat request; pub
  proc pub:faqs_chat {nick uhost hand chan arg} {
    return [faqs_chat pub putmsg $chan putnot $nick $nick $uhost $hand $chan $arg]
  }

  # answer a chat request; msg
  proc msg:faqs_chat {nick uhost hand arg} {
    return [faqs_chat msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  # send dcc chat request
  proc faqs_chat {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
    variable conf
    variable port
    variable NamespaceCurrent
    if {![session_check3 $uhost ${NamespaceCurrent}::control_faqs]} then {
      if {($conf(port)) && ([info exists port])} then {
        putchat $nick $port
      }
    }
    return 1
  }

  # put the idx under control of the script
  proc listen_faqs {idx} {
    variable conf
    variable NamespaceCurrent
    if {[session_check1 ${NamespaceCurrent}::control_faqs] < $conf(maxusers)} then {
      if {[session_check3 [idx2uhost $idx] ${NamespaceCurrent}::control_faqs] <= 1} then {
        putdcc $idx "Entering faq system..."
        control $idx ${NamespaceCurrent}::control_faqs
        putdcc $idx "faqs>"
      } else {
        killdcc $idx
      }
    } else {
      putdcc $idx "Sorry, too many users are using the faq system, please try again later."
      killdcc $idx
    }
    return 0
  }

  # start a faq session if the user is already on the partyline; dcc
  proc dcc:faqs {hand idx arg} {
    variable conf
    variable NamespaceCurrent
    if {[session_check1 ${NamespaceCurrent}::control_faqs] < $conf(maxusers)} then {
      putdcc $idx "Entering faq system..."
      control $idx ${NamespaceCurrent}::control_faqs
      putdcc $idx "faqs>"
    } else {
      putdcc $idx "Sorry, too many users are using the faq system, please try again later."
    }
    return 1
  }

  # interact with the user
  proc control_faqs {idx text} {
    variable conf
    variable faqdata
    variable faqdata_changed
    variable NamespaceCurrent
    set hand [idx2hand $idx]
    if {![validuser $hand]} then {
      set uhost [idx2uhost $idx]
    }
    set text [split [string trimright $text]]
    set command [string trimleft [lindex $text 0] .]
    switch -exact [string tolower $command] {
      list {
        set topics [array names faqdata]
        if {[string compare "" $topics]} then {
          putdcc $idx "Faq topics avaliable:"
          foreach i [lsort $topics] {
            putdcc $idx "  $i"
          }
        } else {
          putdcc $idx "Error: no topics in database."
        }
      }
      show {
        set what [lindex $text 1]
        if {![string compare "" $what]} then {
          putdcc $idx "Usage: show <topic>"
        } else {
          set topic [arrayFindElementName ${NamespaceCurrent}::faqdata $what]
          if {[string compare "" $topic]} then {
            putdcc $idx "Info for $topic: [join $faqdata($topic)]"
          } else {
            putdcc $idx "Faq topic $what does not exist."
          }
        }
      }
      search {
        set what [string trimright [lrange $text 1 end]]
        if {![string compare "" $what]} then {
          putdcc $idx "Usage: search <pattern>"
        } elseif {[string length $what] >= $conf(minsearch)} then {
          set results [array_search ${NamespaceCurrent}::faqdata $what]
          if {[string compare "" $results]} then {
            putdcc $idx "Search results:"
            foreach i $results {
              set topic [lindex $i 0]
              set indexes [lrange $i 1 end]
              set result [join $faqdata($topic)]
              foreach j $indexes {
                set result [lreplace $result $j $j "\002[lindex $result $j]\002"]
              }
              putdcc $idx "$topic: $result"
            }
          } else {
            putdcc $idx "No match for: [join $what]"
          }
        } else {
          putdcc $idx "Search pattern must be at least $conf(minsearch) characters long."
        }
      }
      help {
        set what [lindex $text 1]
        if {![string compare "" $what]} then {
          set what help
        }
        switch -exact $what {
          list {
            putdcc $idx "###  list"
            putdcc $idx "  Lists all faq topics"
          }
          show {
            putdcc $idx "###  show <topic>"
            putdcc $idx "  shows the given faq topic"
          }
          search {
            putdcc $idx "###  search <word>"
            putdcc $idx "  searches all faq topics for the given word"
          }
          help {
            putdcc $idx "Commands for faq system:"
            putdcc $idx "  list"
            putdcc $idx "  show    <topic>"
            putdcc $idx "  search  <word>"
            putdcc $idx "  help   \[command\]"
            putdcc $idx "  quit"
            putdcc $idx "You can get help on individual commands: 'help <command>'"
          }
          quit {
            putdcc $idx "###  quit"
            putdcc $idx "  Quits the faq system and returns you to the partyline."
          }
          default {
            putdcc $idx "No help available on that."
          }
        }
      }
      quit {
        putdcc $idx "*** Ja mata!"
        putdcc $idx "Returning you to command mode..."
        return 1
      }
      default {
        putdcc $idx "What?  You need 'help'"
      }
    }
    if {![info exists uhost]} then {
      wpLog 2 * $NamespaceCurrent "#$hand# $command [lrange $text 1 end]"
    } else {
      wpLog 2 * $NamespaceCurrent "($uhost) $command [lrange $text 1 end]"
    }
    putdcc $idx "faqs>"
    return 0
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "pub msg dcc"
    }
    foreach i $what {
      switch -exact -- $i {
        pub {
          set char [lindex $args 0]
          if {![string compare "" $char]} then {
            set char [commandChar pub]
          }
          $action pub -|- ${char}faq ${NamespaceCurrent}::pub:faq
          $action pub m|- ${char}addfaq ${NamespaceCurrent}::pub:addfaq
          $action pub m|- ${char}delfaq ${NamespaceCurrent}::pub:delfaq
          $action pub m|- ${char}chfaq ${NamespaceCurrent}::pub:chfaq
          $action pub -|- ${char}faqs ${NamespaceCurrent}::pub:faqs_chat
        }
        msg {
          $action msg -|- faq ${NamespaceCurrent}::msg:faq
          $action msg m|- addfaq ${NamespaceCurrent}::msg:addfaq
          $action msg m|- delfaq ${NamespaceCurrent}::msg:delfaq
          $action msg m|- chfaq ${NamespaceCurrent}::msg:chfaq
          $action msg -|- faqs ${NamespaceCurrent}::msg:faqs_chat
        }
        dcc {
          $action dcc -|- faq ${NamespaceCurrent}::dcc:faq
          $action dcc m|- addfaq ${NamespaceCurrent}::dcc:addfaq
          $action dcc m|- delfaq ${NamespaceCurrent}::dcc:delfaq
          $action dcc m|- chfaq ${NamespaceCurrent}::dcc:chfaq
          $action dcc -|- faqs ${NamespaceCurrent}::dcc:faqs
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    faqdatafile faqserv-faq.data
    maxusers 10
    minsearch 3
    port 30000
  }

  # Configuration data descriptions
  array set configDataDesc {
    datafile "\[file\] file that faq data is stored in"
    maxusers "\[number\] max users that can be in faq at once"
    minsearch "\[number\] min number of characters for a search pattern"
    port "\[port\] port to accept telnet and dcc connections on"
  }

  # make sure things are set right when loaded
  if {$conf(port)} then {
    set port [listen $conf(port) script ${NamespaceCurrent}::listen_faqs]
  }

} ;# namespace faqserv

return
