###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: ircrelay.tcl,v 1.4 2001/06/07 08:19:26 tothwolf Exp $
##

# name: ircrelay
# version: 0.4.6.1
# config: ircrelay.conf
# description:
# requires: this_module_needs_to_be_converted
# provides: irc2dcc dcc2irc

namespace eval ircrelay {

  # Set namespace variables
  set NamespaceParent [namespace parent]
  set NamespaceCurrent [namespace current]

  # set data information
  set data(relaydata) "1 $conf(relaydatafile) ${NamespaceCurrent}::relaydata"

  # load data
  module_data load "" 1 0 "1 $conf(relaydatafile) ${NamespaceCurrent}::relaydata"

  # returns dcc channel number associated with the irc channel name
  proc irc2dcc {chan} {
    variable conf
    variable relaydata
    if {!$conf(isolate)} then {
      set chan [string tolower $chan]
      foreach i [array names relaydata] {
        if {![string compare $chan [string tolower $i]]} then {
          return $relaydata($i)
        }
      }
    }
    return
  }

  # returns irc channel name associated with the dcc channel number
  proc dcc2irc {chan} {
    variable conf
    variable relaydata
    if {!$conf(isolate)} then {
      foreach i [array names relaydata] {
        if {![string compare $chan $relaydata($i)]} then {
          return $i
        }
      }
    }
    return
  }

  # tell someone in the irc channel whos in the dcc channel
  proc pub:look {nick uhost hand chan arg} {
    if {[validNick $nick]} then {
      set tmp [irc2dcc $chan]
      if {[string compare "" $tmp]} then {
        foreach i [whom $tmp] {
          lappend list [lindex $i 0],
        }
        if {[info exists list]} then {
          formatLists not $nick [string trimright [join $list] ,] 160 , , " " "Users: " "(cont): "
        } else {
          put not $nick "No users."
        }
      } else {
        put not $nick "$chan is not linked."
      }
      return 1
    }
    return 0
  }

  # tell someone in the dcc channel whos in the irc channel; dcc
  proc dcc:look {hand idx arg} {
    global botnick
    set tmp [dcc2irc [getchan $idx]]
    if {[string compare "" $tmp]} then {
      foreach i [chanlist $tmp)] {
        if {[string compare $botnick $i]} then {
          lappend list $i,
        }
      }
      if {[info exists list]} then {
        formatLists putdcc $idx [string trimright [join $list] ,] 160 , , " " "Users: " "(cont): "
      } else {
        putdcc $idx "No users."
      }
    } else {
      putdcc $idx "$chan is not linked."
    }
    return 1
  }

  # relay irc channel chat to the dcc channel
  proc pubm:relay {nick uhost hand chan text} {
    if {[validNick $nick]} then {
      set tmp [irc2dcc $chan]
      if {[string compare "" $tmp]} then {
        dccputchan $tmp "<<$nick>> $text"
      }
    }
    return 0
  }

  # relay irc channel actions to the dcc channel
  proc ctcp:action_relay {nick uhost hand dest key arg} {
    if {[validNick $nick]} then {
      set tmp [irc2dcc $dest]
      if {[string compare "" $tmp]} then {
        dccputchan $tmp "** $nick $arg"
      }
    }
    return 0
  }

  # relay irc channel joins to the dcc channel
  proc join:relay {nick uhost hand chan} {
    if {[validNick $nick]} then {
      set tmp [irc2dcc $chan]
      if {[string compare "" $tmp]} then {
        dccputchan $tmp "$nick joined $chan"
      }
    }
    return 0
  }

  # relay irc channel parts to the dcc channel
  proc part:relay {nick uhost hand chan} {
    if {[validNick $nick]} then {
      set tmp [irc2dcc $chan]
      if {[string compare "" $tmp]} then {
        dccputchan $tmp "$nick left $chan"
      }
    }
    return 0
  }

  # relay irc channel signoffs to the dcc channel
  proc sign:relay {nick uhost hand chan reas} {
    if {[validNick $nick]} then {
      set tmp [irc2dcc $chan]
      if {[string compare "" $tmp]} then {
        dccputchan $tmp "$nick left irc ($reas)"
      }
    }
    return 0
  }

  # relay irc channel topic changes to the dcc channel
  proc topc:relay {nick uhost hand chan topic} {
    set tmp [irc2dcc $chan]
    if {[string compare "" $tmp]} then {
      dccputchan $tmp "$nick set the topic on $chan to $topic"
    }
    return 0
  }

  # relay irc channel kicks to the dcc channel
  proc kick:relay {nick uhost hand chan knick reas} {
    set tmp [irc2dcc $chan]
    if {[string compare "" $tmp]} then {
      dccputchan $tmp "$knick has been kicked off $chan by $nick ($reas)"
    }
    return 0
  }

  # relay irc channel nick changes to the dcc channel
  proc nick:relay {nick uhost hand chan newnick} {
    set tmp [irc2dcc $chan]
    if {[string compare "" $tmp]} then {
      dccputchan $tmp "$nick is now known as $newnick"
    }
    return 0
  }

  # relay irc channel mode changes to the dcc channel
  proc mode:relay {nick uhost hand chan mode arg} {
    set tmp [irc2dcc $chan]
    if {[string compare "" $tmp]} then {
      if {![string compare "" $arg]} then {
        dccputchan $tmp "$nick set mode \"$mode\" on $chan"
      } else {
        dccputchan $tmp "$nick set mode \"$mode $arg\" on $chan"
      }
    }
    return 0
  }

  # relay dcc channel joins to the irc channel
  proc chjn:relay {bot hand chan flag idx from} {
    set tmp [dcc2irc $chan]
    if {[string compare "" $tmp]} then {
      putdcc $idx "You have entered $tmp"
      putmsg $tmp "\002>>>\002 $hand entered the clearing"
    }
    return 0
  }

  # relay dcc channel parts to the irc channel
  proc chpt:relay {bot hand idx chan} {
    set tmp [dcc2irc $chan]
    if {[string compare "" $tmp]} then {
      putdcc $idx "You have left $tmp"
      putmsg $tmp "\002>>>\002 $hand left the clearing"
    }
    return 0
  }

  # relay dcc channel chat to the irc channel
  proc chat:relay {nick chan text} {
    set tmp [dcc2irc $chan]
    if {[string compare "" $tmp]} then {
      putmsg $tmp "\002<\002$nick\002>\002 $text"
    }
    return 0
  }

  # relay dcc channel actions to the irc channel
  proc act:relay {nick chan text} {
    set tmp [dcc2irc $chan]
    if {[string compare "" $tmp]} then {
      putmsg $tmp "\002*\002 $nick $text"
    }
    return 0
  }

  # relay dcc channel nick changes to the irc channel
  proc nkch:relay {nick newnick} {
    set idx [hand2idx $newnick]
    if {[valididx $idx]} then {
      set tmp [dcc2irc [getchan $idx]]
      if {[string compare "" $tmp]} then {
        putmsg $tmp "\002>>>\002 $nick is now known as $newnick"
      }
    }
    return 0
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "pub dcc pubm ctcp join part sign topc kick nick mode \
	chjn chpt chat act nkch"
    }
    foreach i $what {
      switch -exact -- $i {
        pub {
          set char [lindex $args 0]
          if {![string compare "" $char]} then {
            set char [commandChar pub]
          }
          $action pub -|- ${char}look ${NamespaceCurrent}::pub:look
        }
        dcc {
          $action dcc -|- look ${NamespaceCurrent}::dcc:look
        }
        pubm {
          $action pubm -|- * ${NamespaceCurrent}::pubm:relay
        }
        ctcp {
          $action ctcp -|- ACTION ${NamespaceCurrent}::ctcp:action_relay
        }
        join {
          $action join -|- * ${NamespaceCurrent}::join:relay
        }
        part {
          $action part -|- * ${NamespaceCurrent}::part:relay
        }
        sign {
          $action sign -|- * ${NamespaceCurrent}::sign:relay
        }
        topc {
          $action topc -|- * ${NamespaceCurrent}::topc:relay
        }
        kick {
          $action kick -|- * ${NamespaceCurrent}::kick:relay
        }
        nick {
          $action nick -|- * ${NamespaceCurrent}::nick:relay
        }
        mode {
          $action mode -|- * ${NamespaceCurrent}::mode:relay
        }
        chjn {
          $action chjn -|- * ${NamespaceCurrent}::chjn:relay
        }
        chpt {
          $action chpt -|- * ${NamespaceCurrent}::chpt:relay
        }
        chat {
          $action chat -|- * ${NamespaceCurrent}::chat:relay
        }
        act {
          $action act -|- * ${NamespaceCurrent}::act:relay
        }
        nkch {
          $action nkch -|- * ${NamespaceCurrent}::nkch:relay
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    isolate 0
    relaydatafile ircrelay-chan.data
  }

  # Configuration data descriptions
  array set configDataDesc {
    isolate "\[0/1\] set to 1 to isolate irc <-> dcc traffic"
    relaydatafile "\[file\] file to store relay config data in"
  }

} ;# namespace ircrelay

return
