###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: jester.tcl,v 1.3 2001/06/01 05:38:49 tothwolf Exp $
##

# name: jester
# version: 1.0.7.1
# config: jester.conf
# description:
# requires: this_module_needs_to_be_converted

namespace eval jester {

  # Set namespace variables
  set NamespaceParent [namespace parent]
  set NamespaceCurrent [namespace current]

  # FIXME: set missing varliables to defaults
  if {![info exists chandata_last]} then {set chandata_last ""}
  if {![info exists jokedata_last]} then {set jokedata_last ""}
  if {![info exists listactive]} then {set listactive 0}

  # set data information
  set data(chandata) "1 $conf(chandatafile) ${NamespaceCurrent}::chandata"
  set data(jokedata) "1 $conf(jokedatafile) ${NamespaceCurrent}::jokedata"
  set data(tempdata) "1 $conf(tempdatafile) ${NamespaceCurrent}::tempdata"

  # load data
  module_data load "" 1 0 "1 $conf(chandatafile) ${NamespaceCurrent}::chandata"
  module_data load "" 1 0 "1 $conf(jokedatafile) ${NamespaceCurrent}::jokedata"
  module_data load "" 1 0 "1 $conf(tempdatafile) ${NamespaceCurrent}::tempdata"

  # look for a place to say our joke, say it if we are already on that channel.
  proc annoy_channel {} {
    variable conf
    variable joinchan
    variable NamespaceCurrent
    set chan [random_index ${NamespaceCurrent}::chandata ${NamespaceCurrent}::chandata_last]
    if {[string compare "" $chan]} then {
      if {[validchan $chan]} then {
        set joke [random_index ${NamespaceCurrent}::jokedata ${NamespaceCurrent}::jokedata_last]
        if {[string compare "" $joke]} then {
          tell_joke $chan $joke
        }
      } else {
        if {[add_channel $chan]} then {
          set joinchan $chan
          timer $conf(delay) "${NamespaceCurrent}::remove_channel $chan"
        }
      }
    }
    return 0
  }

  # say our joke when we join a channel
  proc join:tell_joke {nick uhost hand chan} {
    global botnick
    variable joinchan
    variable NamespaceCurrent
    if {(![string compare $botnick $nick]) && \
        (![string compare [string tolower $joinchan] [string tolower $chan]])} then {
      set joke [random_index ${NamespaceCurrent}::jokedata ${NamespaceCurrent}::jokedata_last]
      if {[string compare "" $joke]} then {
        utimer 2 "${NamespaceCurrent}::tell_joke $chan $joke"
      }
      set joinchan ""
    }
    return 0
  }

  # check number of users in a channel and say our joke
  proc tell_joke {chan joke} {
    variable conf
    if {([validchan $chan]) && ([botonchan $chan])} then {
      if {[llength [chanlist $chan]] >= $conf(minusers)} then {
        foreach i $joke {
          putmsg $chan $i
        }
        return 1
      }
    }
    return 0
  }

  # add a channel and set default options if it dosen't exist
  proc add_channel {chan} {
    variable conf
    variable tempdata
    variable tempdata_changed
    if {![validchan $chan]} then {
      lappend tempdata $chan
      set tempdata_changed 1
      channel add $chan
      eval channel set $chan $conf(chanoptions)
      return 1
    }
    return 0
  }

  # remove a channel if it exists
  proc remove_channel {chan} {
    variable tempdata
    variable tempdata_changed
    if {[validchan $chan]} then {
      set index [lsearch -exact $tempdata $chan]
      if {$index != -1} then {
        channel remove $chan
        set tempdata [lreplace $tempdata $index $index]
        set tempdata_changed 1
        return 1
      }
    }
    return 0
  }

  # remove channels we shouldn't be in
  proc check_channels {} {
    variable tempdata
    variable tempdata_changed
    set tempchans [string tolower [join $tempdata]]
    foreach i [string tolower [channels]] {
      set index [lsearch -exact $tempchans $i]
      if {$index != -1} then {
        channel remove $i
        set tempdata [lreplace $tempdata $index $index]
        set tempdata_changed 1
      }
    }
    return 0
  }

  # watch for invite requests
  proc raw:invite {from key arg} {
    variable conf
    variable joinchan
    variable timers
    variable NamespaceCurrent
    set chan [string trimleft [lindex [split $arg] 1] :]
    if {![validchan $chan]} then {
      if {[add_channel $chan]} then {
        set joinchan $chan
        set lchan [string tolower $chan]
        set timers($lchan) [timer $conf(delay) "catch {${NamespaceCurrent}::remove_channel $chan ; unset timers($lchan)}"
      }
    }
    return 0
  }

  # stay if asked
  proc pub:stay {nick uhost hand chan arg} {
    variable timers
    if {[isop $nick $chan]} then {
      set lchan [string tolower $chan]
      if {[info exists timers($lchan)]} then {
        killtimer $timers($lchan)
        unset timers($lchan)
      }
    }
    return 1
  }

  # leave if asked
  proc pub:leave {nick uhost hand chan arg} {
    variable tempdata
    variable tempdata_changed
    if {[isop $nick $chan]} then {
      set index [lsearch -exact $tempdata $chan]
      if {$index != -1} then {
        channel remove $i
        set tempdata [lreplace $tempdata $index $index]
        set tempdata_changed 1
      }
    }
    return 1
  }

  # get a new channel list
  proc get_channels {} {
    variable listactive
    set listactive 1
    putserv "LIST"
    return 0
  }

  # begin of channel list
  proc raw:321 {from key arg} {
    variable listactive
    variable listdata
    if {$listactive} then {
      set listdata ""
    }
    return 0
  }

  # channel list
  proc raw:322 {from key arg} {
    variable listactive
    variable listdata
    if {$listactive} then {
      set chan [lindex [split $arg] 1]
      lappend listdata $chan
    }
    return 0
  }

  # end of channel list
  proc raw:323 {from key arg} {
    variable listactive
    variable listdata
    variable chandata
    variable chandata_changed
    if {$listactive} then {
      set listactive 0
      set chandata ""
      foreach i $listdata {
        set ok 1
        foreach j [string tolower $conf(exclude)] {
          if {[regexp ".*$j.*" [string tolower $i]]} then {
            set ok 0
          }
        }
        if {($ok) && ([string compare "*" $chan]) && \
            ((!$conf(maxlength)) || \
             ([string length $i] <= $conf(maxlength)))} then {
          lappend chandata $i
        }
      }
      unset listdata
      set chandata_changed 1
    }
    return 0
  }

  # find a new channel and annoy it every 5 minutes
  proc time:min_5 {min hour day month year} {
    annoy_channel
    return 0
  }

  # remove channels we shouldn't be in and get a new channel list every 12 hours
  proc time:hour_12 {min hour day month year} {
    check_channels
#    get_channels
    return 0
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "pub join raw time"
    }
    foreach i $what {
      switch -exact -- $i {
        pub {
          set char [lindex $args 0]
          if {![string compare "" $char]} then {
            set char [commandChar pub]
          }
          $action pub -|- ${char}stay ${NamespaceCurrent}::pub:stay
          $action pub -|- ${char}leave ${NamespaceCurrent}::pub:leave
        }
        join {
          $action join -|- * ${NamespaceCurrent}::join:tell_joke
        }
        raw {
          $action raw -|- INVITE ${NamespaceCurrent}::raw:invite
          $action raw -|- 321 ${NamespaceCurrent}::raw:321
          $action raw -|- 322 ${NamespaceCurrent}::raw:322
          $action raw -|- 323 ${NamespaceCurrent}::raw:323
        }
        time {
          $action time -|- "?0 * * * *" ${NamespaceCurrent}::time:min_5
          $action time -|- "?5 * * * *" ${NamespaceCurrent}::time:min_5
          $action time -|- "00 12 * * *" ${NamespaceCurrent}::time:hour_12
          $action time -|- "00 00 * * *" ${NamespaceCurrent}::time:hour_12
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    chandatafile jester-chan.data
    jokedatafile jester-joke.data
    tempdatafile jester-temp.data
    delay 1
    minusers 2
    exclude "! boy daughter dog fuck gay girl hack hax lesbian little monkey mp3 porn sex shell son teen wares warez xxx"
    maxlength 20
    chanoptions "-clearbans -enforcebans +dynamicbans +userbans -autoop -bitch -greet -protectops -statuslog -stopnethack -revenge -secret -shared -autovoice -cycle -seen +dontkickops -wasoptest"
  }

  # Configuration data descriptions
  array set configDataDesc {
    chandatafile "\[file\] file that will store list channel data"
    jokedatafile "\[file\] file that will store joke data"
    tempdatafile "\[file\] file that will store temp channel data"
    delay "\[minutes\] minutes to delay before removing a channel that was added"
    minusers "\[number\] min number of users that must be in channel to tell a joke"
    exclude "\[keywords\] keywords to exclude from channel data"
    maxlength "\[number\] max length of a channel to join"
    chanoptions "\[options\] default options for added channels"
  }

#  # make sure things are set right when loaded
#  if {![llength [join $chandata]]} then {
#    utimer 30 "${NamespaceCurrent}::get_channels"
#  }

} ;# namespace jester

return
