###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: netserv.tcl,v 1.2 2001/06/01 05:38:49 tothwolf Exp $
##

# name: netserv
# version: 1.0.0.0
# config: netserv.conf
# description: Adds the bot's server to the network variable (40 chars max) \
#	so that the `.botinfo' command will display the bot's current server.
# provides: updateNetwork

namespace eval netserv {

  ##
  ## Update $network variable to show bot's current server
  ##
  ## Args:    none
  ## Returns: nothing
  ##
  proc updateNetwork {} {
    global network server
    variable configData

    if {([info exists network]) && ([info exists server])} then {
      if {($configData(keepnetwork)) && \
          ([string compare "" $network])} then {
        set newnetwork [lindex [split $network :] 0]
        if {[string compare "" $server]} then {
          append newnetwork ": [lindex [split $server :] 0]"
        } elseif {[llength $network] >= 2} then {
          append newnetwork ": (no server)"
        }
        if {[string compare $newnetwork $network]} then {
          set network [string range $newnetwork 0 39]
        }
      } else {
        set network [string range $server 0 39]
      }
    }
    return
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    global network server init-server
    variable NamespaceCurrent

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
        # Work around for buggy tcl variables in certain bots
        catch {set network}
        catch {set server}
        if {(![info exists init-server]) || \
            (![string compare "" ${init-server}])} then {
          set init-server ${NamespaceCurrent}::updateNetwork
        } elseif {[lsearch -exact ${init-server} \
                   ${NamespaceCurrent}::updateNetwork] == -1} then {
          append init-server " ; ${NamespaceCurrent}::updateNetwork"
        }
      }
      unload {
        set index [lsearch -exact ${init-server} \
                   ${NamespaceCurrent}::updateNetwork]
        if {$index != -1} then {
          set sindex [expr $index - 1]
          if {[string compare ";" [lindex ${init-server} $sindex]]} then {
            set sindex $index
          }
          set init-server [lreplace ${init-server} $sindex $index ""]
        }
        if {([info exists network]) && ([info exists server])} then {
          if {[string compare "" $network]} then {
            set tmp [split $network :]
            set oldserver [lindex $tmp 1]
            if {(![string compare $oldserver [lindex [split $server :] 0]]) || \
                (![string compare $oldserver "(no server)"])} then {
              set network [lindex $tmp 0]
            }
          }
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    keepnetwork 1
  }

  # Configuration data descriptions
  array set configDataDesc {
    keepnetwork "\[0/1\] keep irc network name in the network var when adding server name?"
  }

} ;# namespace netserv

return
