###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: portctrl.tcl,v 1.3 2001/06/07 08:19:26 tothwolf Exp $
##

# name: portctrl
# version: 1.2.0.0
# config: portctrl.conf
# description: Adds a dcc command called `.portctrl' that can be used to \
#	enable/disable the bot's listen port(s).
# requires: listListenPorts timerId
# provides: portControl

namespace eval portctrl {

  ##
  ## Enable/disable listen ports: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:portctrl {hand idx arg} {
    variable configData

    set arg [string trim [lindex [split $arg] 0]]
    set types [join [lrange $arg 1 end]]
    if {![string compare "" $types]} then {
      set types $configData(types)
    }
    switch -exact -- [string tolower [lindex $arg 0]] {
      enable {
        portControl enable $types
        putdcc $idx "Listen port types enabled: $types"
      }
      disable {
        portControl disable $types
        putdcc $idx "Listen port types disabled: $types"
      }
      default {
        putdcc $idx "Usage: portctrl <enable|disable> \[port types\]"
        putdcc $idx "Default types: $configData(types)"
      }
    }
    return 1
  }

  ##
  ## Enable/disable listen ports
  ##
  ## Args:    mode
  ## Returns: nothing
  ##
  proc portControl {mode {types ""}} {
    variable NamespaceCurrent
    variable configData
    variable DisabledPorts

    if {![string compare "" $types]} then {
      set types $configData(types)
    }
    switch -exact -- $mode {
      enable {
        if {[info exists DisabledPorts]} then {
          if {($configData(timeout)) && \
              ([string compare "" [set timerid [timerId "${NamespaceCurrent}::portControl enable" utimers]]])} then {
            killutimer $timerid
          }
          foreach list $DisabledPorts {
            foreach {port type mask} $list {break}
            if {[info exists mask]} then {
              listen $port $type $mask
            } else {
              listen $port $type
            }
          }
          unset DisabledPorts
        }
      }
      disable {
        foreach list [listListenPorts] {
          if {([lsearch -exact $DisabledPorts $list] == -1) && \
              ([lsearch -exact $types [lindex $list 1]] != -1)} then {
            lappend DisabledPorts $list
            listen [lindex $list 0] off
          }
        }
        if {$configData(timeout)} then {
          if {[string compare "" [set timerid [timerId "${NamespaceCurrent}::portControl enable" utimers]]]} then {
            killutimer $timerid
          }
          utimer $configData(timeout) "${NamespaceCurrent}::portControl enable"
        }
      }
    }
    return
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable NamespaceCurrent
    variable DisabledPorts

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
        if {![info exists DisabledPorts]} then {
          set DisabledPorts ""
        }
      }
    }
    return
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what dcc
    }
    foreach type $what {
      switch -exact -- $type {
        dcc {
          $action dcc n|- portctrl ${NamespaceCurrent}::dcc:portctrl
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    types "bots all"
    timeout 600
  }

  # Configuration data descriptions
  array set configDataDesc {
    types "\[types\] what listen port types should be disabled?"
    timeout "\[seconds\] reenable listen ports after how many seconds? (0 to disable)"
  }

} ;# namespace portctrl

return
