###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: showall.tcl,v 1.3 2001/06/06 06:15:30 tothwolf Exp $
##

# name: showall
# version: 0.8.0.0
# description: Displays output simmilar to `.channel' for all channels or a \
#	list of channels that the bot monitors.
# requires: botonchan duration formatChanJoin formatChanIdle chanFlag

namespace eval showall {

  ##
  ## Show all user info for the channels given, or all channels
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:showall {hand idx arg} {
    set chans [string trimright $arg]
    if {[string compare "" $chans]} then {
      set channels $chans
    } else {
      set channels [lsort [channels]]
    }
    if {[string compare "" $channels]} then {
      foreach chan $channels {
        if {[validchan $chan]} then {
          if {![botonchan $chan]} then {
            putdcc $idx "Desiring channel $chan, 0 members, mode +:"
            # putdcc can't handle ""
            putdcc $idx " "
          } else {
            set chanlist [chanlist $chan]
            putdcc $idx "Channel $chan, [llength [split $chanlist]] members, mode [getchanmode $chan]:"
            putdcc $idx "(n = owner, m = master, o = op, d = deop, b = bot)"
            # putdcc can't handle ""
            putdcc $idx " "
            putdcc $idx " NICKNAME        HANDLE    JOIN   IDLE  USER@HOST"
            foreach nick $chanlist {
              if {[isop $nick $chan]} then {
                set display(status) @
              } elseif {[isvoice $nick $chan]} then {
                set display(status) +
              } else {
                set display(status) " "
              }
              set display(nick) $nick
              set chanjoin [getchanjoin $nick $chan]
              if {$chanjoin} then {
                set display(join) [formatChanJoin [getchanjoin $nick $chan]]
              } else {
                set display(join) ---
              }
              set chanhand [nick2hand $nick $chan]
              if {[validuser $chanhand]} then {
                set display(handle) $chanhand
                set display(flags) [chanFlag $chanhand $chan]
              } else {
                set display(handle) *
                set display(flags) ""
              }
              set chanidle [getchanidle $nick $chan]
              if {$chanidle <= 1} then {
                set display(idle) ""
              } else {
                set display(idle) [formatChanIdle $chanidle]
              }
              if {[isbotnick $nick]} then {
                set display(host) "<- it's me!"
              } elseif {[onchansplit $nick $chan]} {
                set laston [getuser $nick LASTON $chan]
                if {$laston} then {
                  set display(host) "<- netsplit, [duration [expr [clock seconds] - $laston]]"
                } else {
                  set display(host) "<- netsplit, unknown"
                }
              } else {
                set display(host) [getchanhost $nick $chan]
              }
              putdcc $idx [format "%s%-16s%-10s%-6s%s%4s  %s" $display(status) $display(nick) $display(handle) $display(join) $display(flags) $display(idle) $display(host)]
            }
            # putdcc can't handle ""
            putdcc $idx " "
          }
        }
      }
      putdcc $idx "End of channel info."
    } else {
      putdcc $idx "I'm not monitoring any channels."
    }
    return 1
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable NamespaceCurrent

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
      }
    }
    return
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what dcc
    }
    foreach type $what {
      switch -exact -- $type {
        dcc {
          $action dcc m|- showall ${NamespaceCurrent}::dcc:showall
        }
      }
    }
    return
  }

} ;# namespace showall

return
