###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: dice.tcl,v 1.6 2001/06/06 06:09:18 tothwolf Exp $
##

# name: wrpg-dice
# version: 1.5.0.0
# config: wrpg-dice.conf
# description: Wolf RPG dice roll command and functions.
# requires: commandChar putmsg isNumber floodCheck
# provides: diceConfig

namespace eval wrpg-dice {

  ##
  ## Dice roll
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1
  ##
  proc pub:roll {nick uhost hand chan arg} {
    variable DiceConfig

    if {[floodCheck $nick $uhost roll]} then {
      if {[validuser $hand]} then {
        set user $hand
      } else {
        set user $nick
      }
      set count 0
      while {$count < $DiceConfig(dice)} {
        set result [expr [rand $DiceConfig(faces)] + 1]
        if {($result >= $DiceConfig(low1_1)) && ($result <= $DiceConfig(low1_2)) && ($DiceConfig(lowodds1_1))} then {
          if {[expr ([rand $DiceConfig(lowodds1_2)] + 1) != $DiceConfig(lowodds1_1)]} then {
            continue
          }
        } elseif {($result >= $DiceConfig(low2_1)) && ($result <= $DiceConfig(low2_2)) && ($DiceConfig(lowodds2_1))} then {
          if {[expr ([rand $DiceConfig(lowodds2_2)] + 1) != $DiceConfig(lowodds2_1)]} then {
            continue
          }
        } elseif {($result >= $DiceConfig(high1_1)) && ($result <= $DiceConfig(high1_2)) && ($DiceConfig(highodds1_1))} then {
          if {[expr ([rand $DiceConfig(highodds1_2)] + 1) != $DiceConfig(highodds1_1)]} then {
            continue
          }
        } elseif {($result >= $DiceConfig(high2_1)) && ($result <= $DiceConfig(high2_2)) && ($DiceConfig(lowodds2_1))} then {
          if {[expr ([rand $DiceConfig(highodds2_2)] + 1) != $DiceConfig(highodds2_1)]} then {
            continue
          }
        }
        lappend results $result
        incr count
      }
      if {([string length results]) && \
          ((![string length $DiceConfig(use)]) || \
           (![$DiceConfig(use) $nick $uhost $hand $chan $results]))} then {
        foreach result $results {
          lappend reply "$DiceConfig(bold)$result$DiceConfig(bold),"
        }
        putmsg $chan "$user rolls a [string trimright $reply ,]"
      }
      return 1
    }
    return 0
  }

  ##
  ## Configure dice settings and values
  ##
  ## Args:    what {reset|use|config}, args {options|command}
  ## Returns: nothing
  ##
  proc diceConfig {what args} {
    variable configData
    variable DiceConfig

    set options [lindex $args 0]
    switch -exact -- $what {
      reset {
        if {$configData(bold)} then {
          set DiceConfig(bold) "\002"
        } else {
          set DiceConfig(bold) ""
        }
        set DiceConfig(use) ""
        set DiceConfig(dice) $configData(dice)
        set DiceConfig(faces) $configData(faces)
        foreach option "low1 low2 high1 high2 lowodds1 lowodds2 highodds1 highodds2" {
          foreach "DiceConfig(${option}_1) DiceConfig(${option}_2)" [split $configData($option) :] {break}
        }
      }
      use {
        set DiceConfig(use) $options
      }
      config {
        if {([llength $options] == 16) && ([isNumber [join $options ""]])} then {
          foreach "DiceConfig(dice) DiceConfig(faces) \
                   DiceConfig(low1_1) DiceConfig(low1_2) \
                   DiceConfig(low2_1) DiceConfig(low2_2) \
                   DiceConfig(high1_1) DiceConfig(high1_2) \
                   DiceConfig(high2_1) DiceConfig(high2_2) \
                   DiceConfig(lowodds1_1) DiceConfig(lowodds1_2) \
                   DiceConfig(lowodds2_1) DiceConfig(lowodds2_2) \
                   DiceConfig(highodds1_1) DiceConfig(highodds1_2) \
                   DiceConfig(highodds2_1) DiceConfig(highodds2_2)" $options {break}
        }
      }
    }
    return
  }

  ##
  ## Trace configuration changes
  ##
  ## Args:    name1, name2, operation
  ## Returns: nothing
  ##
  proc TraceConfigData {name1 name2 op} {
    variable configData
    variable oldConfigData
    variable DiceConfig

    if {[string compare $oldConfigData($name2) $configData($name2)]} then {
      switch -exact -- $name2 {
        bold {
          if {$configData($name2)} then {
            set DiceConfig(bold) "\002"
          } else {
            set DiceConfig(bold) ""
          }
        }
        dice {
          set DiceConfig(dice) $configData(dice)
        }
        faces {
          set DiceConfig(faces) $configData(faces)
        }
        high1 {
          scan $configData(high1) "%\[^:\]:%s" DiceConfig(high1_1) DiceConfig(high1_2)
        }
        high2 {
          scan $configData(high2) "%\[^:\]:%s" DiceConfig(high2_1) DiceConfig(high2_2)
        }
        highodds1 {
          scan $configData(highodds1) "%\[^:\]:%s" DiceConfig(lowodds1_1) DiceConfig(lowodds1_2)
        }
        highodds2 {
          scan $configData(highodds2) "%\[^:\]:%s" DiceConfig(lowodds2_1) DiceConfig(lowodds2_2)
        }
        low1 {
          scan $configData(low1) "%\[^:\]:%s" DiceConfig(low1_1) DiceConfig(low1_2)
        }
        low2 {
          scan $configData(low2) "%\[^:\]:%s" DiceConfig(low2_1) DiceConfig(low2_2)
        }
        lowodds1 {
          scan $configData(lowodds1) "%\[^:\]:%s" DiceConfig(highodds1_1) DiceConfig(highodds1_2)
        }
        lowodds2 {
          scan $configData(lowodds2) "%\[^:\]:%s" DiceConfig(highodds2_1) DiceConfig(highodds2_2)
        }
      }
      set oldConfigData($name2) $configData($name2)
    }
    return
  }

  ##
  ## Trace and respond to variable changes
  ##
  ## Args:    action {variable|vdelete}
  ## Returns: nothing
  ##
  proc TraceTable {action} {
    variable NamespaceCurrent
    variable configData

    trace $action configData w ${NamespaceCurrent}::TraceConfigData
    return
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    global network server init-server
    variable NamespaceCurrent
    variable configData
    variable oldConfigData

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
        foreach {name data} [array get configData] {
          if {![info exists oldConfigData($name)]} then {
            set oldConfigData($name) $data
          }
        }
        diceConfig reset
      }
    }
    return
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what pub
    }
    foreach type $what {
      switch -exact -- $type {
        pub {
          set char [lindex $args 0]
          if {![string compare "" $char]} then {
            set char [commandChar pub]
          }
          $action pub -|- ${char}roll ${NamespaceCurrent}::pub:roll
          $action pub -|- ,, ${NamespaceCurrent}::pub:roll
        }
      }
    }
    return
  }

  # Configuration data defaults
  array set configDataDefaults {
    bold 1
    dice 2
    faces 20
    low1 1:1
    low2 2:2
    high1 15:19
    high2 20:20
    lowodds1 10:30
    lowodds2 5:20
    highodds1 0:0
    highodds2 1:5
  }

  # Configuration data descriptions
  array set configDataDesc {
    bold "\[0/1\] display dice results in bold text"
    dice "\[number\] how many dice to roll"
    faces "\[number\] faces on each die"
    low1 "\[low:high\] limits for low1 roll"
    low2 "\[low:high\] limits for low2 roll"
    high1 "\[low:high\] limits for high1 roll"
    high2 "\[low:high\] limits for high2 roll"
    lowodds1 "\[x:range\] chance for reroll for x out of range"
    lowodds2 "\[x:range\] chance for reroll for x out of range"
    highodds1 "\[x:range\] chance for reroll for x out of range"
    highodds2 "\[x:range\] chance for reroll for x out of range"
  }

} ;# namespace wrpg-dice

return
