###############################################################################
## This file is part of the Wolfpack module system
## Copyright (C) 1998-2001  Tothwolf <tothwolf@concentric.net>
## and is distributed according to the GNU general public license.
## For full details, read the top of wolfpack.tcl or the file called
## COPYING that was distributed with this code.
##
## $Id: foodcmds.tcl,v 1.1 2001/06/02 03:15:07 tothwolf Exp $
##

# name: wrpg-foodcmds
# version: 1.0.0.0
# description:
# requires: foodData commandChar

namespace eval wrpg-foodcmds {

  ##
  ## Add the given animal with optional amount to the food list: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:addfood {nick uhost hand chan arg} {
    return [addfood pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
  }

  ##
  ## Add the given animal with optional amount to the food list: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:addfood {nick uhost hand arg} {
    return [addfood msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  ##
  ## Add the given animal with optional amount to the food list: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:addfood {hand idx arg} {
    return [addfood dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  ##
  ## Add the given animal with optional amount to the food list
  ##
  ## Args:    type, resp1, dest1, resp2, dest2, nick, user@host, handle,
  ##          channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc addfood {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
#    if {[authCheck $type addfood $nick $uhost $hand $chan]} then {
#      set arg [split [string trimright $arg]]
#      if {[isNumber [lindex $arg end]]} then {
#        set name [join [lrange $arg 0 [expr [llength $arg] - 2]]]
#        set amount [lindex $arg end]
#      } else {
#        set name [join $arg]
#      }
#      if {![string length $name]} then {
#        put $resp2 $dest2 "Usage: [commandChar $type]addfood <name> \[amount\]"
#      } else {
#        foreach {status realname} [foodData del $name $amount] {break}
#        switch -exact -- $status {
#          2 -
#          1 {
#            put $resp1 $dest1 "Added $realname \[$amount\] to the food list."
#          }
#          0 {
#            put $resp2 $dest2 "I don't know what [grammarA $name] is."
#          }
#          -1 {
#            put $resp2 $dest2 "Invalid amount of $realname to add."
#          }
#        }
#      }
#      return 1
#    }
#    return 0
  }

  ##
  ## Remove the given animal or amount from the food list: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:delfood {nick uhost hand chan arg} {
    return [delfood pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
  }

  ##
  ## Remove the given animal or amount from the food list: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:delfood {nick uhost hand arg} {
    return [delfood msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  ##
  ## Remove the given animal or amount from the food list: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:delfood {hand idx arg} {
    return [delfood dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  ##
  ## Remove the given animal or amount from the food list
  ##
  ## Args:    type, resp1, dest1, resp2, dest2, nick, user@host, handle,
  ##          channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc delfood {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
#    if {[authCheck $type delfood $nick $uhost $hand $chan]} then {
#      set arg [split [string trimright $arg]]
#      if {[isNumber [lindex $arg end]]} then {
#        set name [join [lrange $arg 0 [expr [llength $arg] - 2]]]
#        set amount [lindex $arg end]
#      } else {
#        set name [join $arg]
#      }
#      if {![string length $name]} then {
#        put $resp2 $dest2 "Usage: [commandChar $type]delfood <name> \[amount\]"
#      } else {
#        foreach {status realname} [foodData del $name $amount] {break}
#        switch -exact -- $status {
#          2 {
#            put $resp1 $dest1 "Removed $realname from the food list."
#          }
#          1 {
#            put $resp1 $dest1 "Removed $realname \[$amount\] from the food list."
#          }
#          0 {
#            put $resp2 $dest2 "I don't know what [grammarA $name] is."
#          }
#          -1 {
#            put $resp2 $dest2 "Invalid amount of $realname to remove."
#          }
#          -2 {
#            put $resp2 $dest2 "There is no $realnaimal in the food list."
#          }
#        }
#      }
#      return 1
#    }
#    return 0
  }

  ##
  ## Change the amount for a given animal in the food list: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:chfood {nick uhost hand chan arg} {
    return [chfood pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
  }

  ##
  ## Change the amount for a given animal in the food list: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:chfood {nick uhost hand arg} {
    return [chfood msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  ##
  ## Change the amount for a given animal in the food list: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:chfood {hand idx arg} {
    return [chfood dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  ##
  ## Change the amount for a given animal in the food list
  ##
  ## Args:    type, resp1, dest1, resp2, dest2, nick, user@host, handle,
  ##          channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc chfood {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
#    if {[authCheck $type chfood $nick $uhost $hand $chan]} then {
#      set arg [split [string trimright $arg]]
#      set name [join [lrange $arg 0 [expr [llength $arg] - 2]]]
#      set amount [lindex $arg end]
#      if {(![string compare "" $name]) || (![string compare "" $amount]) || \
#          (![isNumber $amount])} then {
#        put $resp2 $dest2 "Usage: [commandChar $type]chfood <name> <amount>"
#      } else {
#        foreach {status realname} [foodData set $name $amount] {break}
#        switch -exact -- $status {
#          1 {
#            put $resp1 $dest1 "Set $realname amount to \[$amount\] in the food list."
#          }
#          0 {
#            put $resp2 $dest2 "I don't know what [grammarA $name] is."
#          }
#          -1 {
#            put $resp2 $dest2 "Invalid amount to set $realname to."
#          }
#          -2 {
#            put $resp2 $dest2 "There is no $realnaimal in the food list."
#          }
#        }
#      }
#      return 1
#    }
#    return 0
  }

  ##
  ## Display the food list: pub
  ##
  ## Args:    nick, user@host, handle, channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc pub:listfood {nick uhost hand chan arg} {
    return [listfood pub putnot $nick putnot $nick $nick $uhost $hand $chan $arg]
  }

  ##
  ## Display the food list: msg
  ##
  ## Args:    nick, user@host, handle, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc msg:listfood {nick uhost hand arg} {
    return [listfood msg putnot $nick putnot $nick $nick $uhost $hand "" $arg]
  }

  ##
  ## Display the food list: dcc
  ##
  ## Args:    handle, idx, arg
  ## Returns: 1
  ##
  proc dcc:listfood {hand idx arg} {
    return [listfood dcc putdcc $idx putdcc $idx "" "" $hand "" $arg]
  }

  ##
  ## Display the food list
  ##
  ## Args:    type, resp1, dest1, resp2, dest2, nick, user@host, handle,
  ##          channel, arg
  ## Returns: 1 if user is auth'd
  ##          0 otherwise
  ##
  proc listfood {type resp1 dest1 resp2 dest2 nick uhost hand chan arg} {
#    if {[authCheck $type listfood $nick $uhost $hand $chan]} then {
#      set what [split [string trimright $arg]]
#      foreach {status data} [foodData list $what] {break}
#      switch -exact -- $status {
#        3 {
#          put $resp1 $dest1 "Info for $name: [lrange $data 1 end]"
#        }
#        2 {
#          putList $resp1 $dest1 [splitList [lrange $data 2 end] 160 "," ", "] "The following animals are on the food list have $what days left ([lindex $data 1]): " "(cont): "
#        }
#        1 {
#          putList $resp1 $dest1 [splitList [lrange $data 2 end] 160 "," ", "] "The following animals are on the food list ([lindex $data 1]): " "(cont): "
#        }
#        0 {
#          put $resp2 $dest2 "I don't know what [grammarA $name] is."
#        }
#        -1 {
#          put $resp1 $dest1 "The food list is empty :("
#        }
#        -2 {
#          put $resp2 $dest2 "There is no food that has $what days left in the food list."
#        }
#        -3 {
#          put $resp1 $dest1 "Food will not spoil."
#        }
#        -4 {
#          put $resp2 $dest2 "There is no $data in the food list."
#        }
#      }
#      return 1
#    }
#    return 0
  }

  ##
  ## Init module specific things at load time
  ##
  ## Args:    action {load|unload}
  ## Returns: nothing
  ##
  proc InitTable {action} {
    variable NamespaceCurrent

    switch -exact -- $action {
      load {
        if {![info exists NamespaceCurrent]} then {
          set NamespaceCurrent [namespace current]
        }
      }
    }
    return
  }

  ##
  ## Bind procs to commands and events
  ##
  ## Args:    action {bind|unbind}, what {type|*}, args {...}
  ## Returns: nothing
  ##
  proc BindTable {action what args} {
    variable NamespaceCurrent

    if {![string compare * $what]} then {
      set what "pub msg dcc time"
    }
    foreach type $what {
      switch -exact -- $type {
        pub {
          set char [lindex $args 0]
          if {![string compare "" $char]} then {
            set char [commandChar pub]
          }
          $action pub m|m ${char}addfood ${NamespaceCurrent}::pub:addfood
          $action pub m|m ${char}delfood ${NamespaceCurrent}::pub:delfood
          $action pub m|m ${char}chfood ${NamespaceCurrent}::pub:chfood
          $action pub m|m ${char}listfood ${NamespaceCurrent}::pub:listfood
        }
        msg {
          $action msg m|m addfood ${NamespaceCurrent}::msg:addfood
          $action msg m|m delfood ${NamespaceCurrent}::msg:delfood
          $action msg m|m chfood ${NamespaceCurrent}::msg:chfood
          $action msg m|m listfood ${NamespaceCurrent}::msg:listfood
        }
        dcc {
          $action dcc m|m addfood ${NamespaceCurrent}::dcc:addfood
          $action dcc m|m delfood ${NamespaceCurrent}::dcc:delfood
          $action dcc m|m chfood ${NamespaceCurrent}::dcc:chfood
          $action dcc m|m listfood ${NamespaceCurrent}::dcc:listfood
        }
        time {
#          $action time -|- "00 * * * *" ${NamespaceCurrent}::time:hourly
        }
      }
    }
    return
  }

} ;# wrpg-foodcmds

return
